import unittest

from pantomime import parse_mimetype, normalize_mimetype, DEFAULT
from pantomime import useful_mimetype


class MIMETest(unittest.TestCase):
    def test_normalize(self):
        self.assertEqual(normalize_mimetype("TEXT/ PLAIN"), "text/plain")
        self.assertEqual(normalize_mimetype("TEXT/"), DEFAULT)
        self.assertEqual(normalize_mimetype("1"), DEFAULT)
        self.assertEqual(normalize_mimetype("1", default=None), None)
        self.assertEqual(normalize_mimetype(None), DEFAULT)

        PST = "application/VND.ms-outlook"
        self.assertEqual(normalize_mimetype(PST), PST.lower())

    def test_useful(self):
        self.assertFalse(useful_mimetype(None))
        self.assertFalse(useful_mimetype(DEFAULT))
        self.assertTrue(useful_mimetype("image/png"))

    def test_label(self):
        parsed = parse_mimetype("application/x-pudo-banana")
        self.assertEqual(parsed.label, "pudo banana")

    def test_parse(self):
        parsed = parse_mimetype("text/plain")
        self.assertEqual(parsed.charset, None)
        self.assertEqual(parsed.label, "Plain text")
        self.assertEqual(parsed.family, "text")
        self.assertEqual(parsed.subtype, "plain")
        self.assertEqual(parsed.normalized, "text/plain")
        self.assertEqual("%s" % parsed, "text/plain")
        self.assertEqual("%r" % parsed, "text/plain")
        parsed = parse_mimetype("text/plain; charset=cp1251")
        self.assertEqual(parsed.charset, "cp1251")
        parsed = parse_mimetype("text/plain; charset=banana")
        self.assertEqual(parsed.charset, "utf-8")

        self.assertEqual(parsed, parse_mimetype("text/plain"))

    def test_parse_rewrite(self):
        parsed = parse_mimetype("plain/text")
        self.assertEqual(parsed.normalized, "text/plain")



from pprint import pprint
from collections import defaultdict
import csv

from pantomime import parse_mimetype


data = defaultdict(int)
with open('occrp.csv', 'r') as fh:
    reader = csv.reader(fh, delimiter=';')
    for row in reader:
        original, count = row
        parsed = parse_mimetype(original)
        print(parsed.label)
        # data[parsed.normalized] += int(count)
        # if parsed.normalized != original:
        #     pprint((original, parsed.label))


print(len(data))



from email.message import EmailMessage
from typing import Any, Dict, Optional, Tuple
from normality import stringify
from normality.encoding import tidy_encoding

from pantomime.types import DEFAULT, LABELS
from pantomime.mappings import REPLACE


class MIMEType(object):
    __slots__ = ["family", "subtype", "params", "name", "normalized"]

    SEP = "/"

    def __init__(
        self,
        family: Optional[str],
        subtype: Optional[str],
        params: Optional[Dict[str, str]] = None,
    ):
        self.family = family
        self.subtype = subtype
        self.name: Optional[str] = None
        if self.family is not None and self.subtype is not None:
            self.name = self.SEP.join((self.family, self.subtype))
        self.normalized: Optional[str] = self.name
        if self.name in REPLACE:
            self.normalized = REPLACE.get(self.name, self.name)
        self.params: Dict[str, str] = params or {}

    @property
    def label(self) -> Optional[str]:
        if self.normalized in LABELS:
            return LABELS.get(self.normalized, self.normalized)
        if self.subtype is not None:
            label = self.subtype.lstrip("x")
            label = label.replace("-", " ")
            label = label.replace(".", " ")
            return label.strip()
        return None

    @property
    def charset(self) -> Optional[str]:
        charset = self.params.get("charset")
        if charset is None:
            return None
        return tidy_encoding(charset)

    @classmethod
    def split(cls, mime_type: Optional[str]) -> Tuple[Optional[str], Optional[str]]:
        if mime_type is None or cls.SEP not in mime_type:
            return None, None
        family, subtype = (p.strip() for p in mime_type.split(cls.SEP, 1))
        if len(family) == 0 or len(subtype) == 0:
            return None, None
        return family.lower(), subtype.lower()

    @classmethod
    def parse(
        cls, mime_type: Optional[str], default: Optional[str] = None
    ) -> "MIMEType":
        mime_type = stringify(mime_type)
        params = None
        if mime_type is not None:
            msg = EmailMessage()
            msg['content-type'] = mime_type
            mime_type = msg.get_content_type() if mime_type.count("/") == 1 else None
            params = msg['content-type'].params

        family, subtype = cls.split(mime_type)
        if family is None:
            family, subtype = cls.split(default)
        return cls(family, subtype, params=params)

    def __eq__(self, other: Any) -> bool:
        return str(self) == str(other)

    def __hash__(self) -> int:
        return hash(str(self))

    def __str__(self) -> str:
        return self.name or DEFAULT

    def __repr__(self) -> str:
        return str(self)



from pantomime.parse import MIMEType
from pantomime.types import DEFAULT, PLAIN
from pantomime.mime import parse_mimetype, normalize_mimetype
from pantomime.mime import useful_mimetype
from pantomime.filename import FileName
from pantomime.filename import normalize_extension, mimetype_extension

__all__ = [
    "MIMEType",
    "FileName",
    "DEFAULT",
    "PLAIN",
    "parse_mimetype",
    "normalize_mimetype",
    "useful_mimetype",
    "normalize_extension",
    "mimetype_extension",
]



from pantomime.util import gettext

DEFAULT = "application/octet-stream"
DIRECTORY = "inode/directory"
EMPTY = "inode/x-empty"

PLAIN = "text/plain"
PDF = "application/pdf"
EXCEL = "application/vnd.ms-excel"
XLS = EXCEL
XLSX = "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"
WORD = "application/vnd.ms-word"
DOCX = "application/vnd.openxmlformats-officedocument.wordprocessing"
CSV = "text/csv"
RTF = "text/rtf"
XML = "text/xml"
PSD = "image/vnd.adobe.photoshop"
RAR = "application/rar"
ZIP = "application/zip"
GZIP = "application/gzip"
RFC822 = "message/rfc822"
HTML = "text/html"
JPEG = "image/jpeg"
PNG = "image/png"
GIF = "image/gif"
TIFF = "image/tiff"
DJVU = "image/x.djvu"
OPF = "application/xml+opfmessage"
OUTLOOK = "application/vnd.ms-outlook"
ZIP = "application/zip"
JSON = "application/json"
FTM = "application/json+ftm"
FTM_RSLV = "application/json+ftm-rslv"
FTM_STMT = "application/json+ftm-statements"

MIXED = "multipart/mixed"
ALTERNATIVE = "multipart/alternative"
RELATED = "multipart/related"


LABELS = {
    DEFAULT: gettext("Unknown file type"),
    DIRECTORY: gettext("Directory"),
    EMPTY: gettext("Empty file"),
    PLAIN: gettext("Plain text"),
    PDF: gettext("Portable Document Format"),
    EXCEL: gettext("Microsoft Excel"),
    XLSX: gettext("Microsoft Excel 2002+"),
    WORD: gettext("Microsoft Word"),
    DOCX: gettext("Microsoft Word 2002+"),
    CSV: gettext("Comma-separated table"),
    RTF: gettext("Rich text"),
    PSD: gettext("Adobe Photoshop"),
    RAR: gettext("WinRAR archive"),
    ZIP: gettext("Zip archive"),
    GZIP: gettext("GZip archive"),
    RFC822: gettext("Plain E-Mail"),
    HTML: gettext("HTML Web Page"),
    JPEG: gettext("JPEG Image"),
    TIFF: gettext("Tagged Image File Format"),
    DJVU: gettext("DejaVu E-Book"),
    PNG: gettext("Portable Network Graphics"),
    GIF: gettext("Graphics Interchange Format"),
    OPF: gettext("Microsoft Outlook for Mac E-Mail"),
    OUTLOOK: gettext("Microsoft Outlook E-Mail"),
    JSON: gettext("JavaScript Object Notation"),
    XML: gettext("eXtensible Markup Language"),
    FTM: gettext("FollowTheMoney Entities"),
    FTM_RSLV: gettext("FollowTheMoney Integrated"),
    FTM_STMT: gettext("FollowTheMoney Statements"),
}



import os
from typing import Any, Optional
from banal import decode_path
from mimetypes import guess_extension
from normality import slugify, safe_filename

from pantomime.mime import normalize_mimetype
from pantomime.types import DEFAULT


def normalize_extension(extension: Optional[str]) -> Optional[str]:
    """Normalise a file name extension."""
    extension = decode_path(extension)
    if extension is None:
        return None
    if extension.startswith("."):
        extension = extension[1:]
    if "." in extension:
        _, extension = os.path.splitext(extension)
    extension = slugify(extension, sep="")
    if extension is None or not len(extension):
        return None
    return extension


def mimetype_extension(mime_type: Optional[str]) -> Optional[str]:
    """Infer a possible extension from a MIME type."""
    mime_type = normalize_mimetype(mime_type)
    if mime_type == DEFAULT:
        return None
    extension = guess_extension(mime_type)
    return normalize_extension(extension)


class FileName(object):
    FALLBACK = "data"

    def __init__(self, file_name: Optional[str]):
        self.file_name = file_name
        self.base: Optional[str] = None
        self.extension: Optional[str] = None
        if file_name is not None:
            self.base, ext = os.path.splitext(file_name)
            self.extension = normalize_extension(ext)
        self.has_extension = self.extension is not None

    def safe(self, extension: Optional[str] = None) -> Optional[str]:
        ext = extension or self.extension
        default = "data.%s" % ext if ext else self.FALLBACK
        return safe_filename(self.file_name, default=default, extension=ext)

    def __str__(self) -> str:
        return self.file_name or self.FALLBACK

    def __repr__(self) -> str:
        return "<FileName(%r)" % self.safe()



from typing import Optional
from pantomime.parse import MIMEType
from pantomime.types import DEFAULT, PLAIN


def parse_mimetype(text: Optional[str], default: str = DEFAULT) -> MIMEType:
    """Parse a MIME type into a structured object."""
    return MIMEType.parse(text, default=default)


def normalize_mimetype(text: Optional[str], default: str = DEFAULT) -> str:
    """Normalize the spelling of a MIME type."""
    return parse_mimetype(text, default=default).normalized or default


def useful_mimetype(text: Optional[str]) -> bool:
    """Check to see if the given mime type is a MIME type
    which is useful in terms of how to treat this file.
    """
    mimetype = normalize_mimetype(text)
    return mimetype not in [DEFAULT, PLAIN, None]



import unittest

from pantomime.filename import normalize_extension
from pantomime.filename import mimetype_extension as mime_ext


class ExtensionTest(unittest.TestCase):
    def test_normalize(self):
        self.assertEqual(normalize_extension(".doc"), "doc")
        self.assertEqual(normalize_extension(None), None)
        self.assertEqual(normalize_extension(""), None)
        self.assertEqual(normalize_extension("bla.doc"), "doc")
        self.assertEqual(normalize_extension("bla.DOC"), "doc")
        self.assertEqual(normalize_extension("bla.DO C"), "doc")
        self.assertEqual(normalize_extension("bla.  DOC  "), "doc")

        self.assertEqual(normalize_extension("TXT"), "txt")
        self.assertEqual(normalize_extension(".TXT"), "txt")
        self.assertEqual(normalize_extension("foo.txt"), "txt")
        self.assertEqual(normalize_extension("foo..TXT"), "txt")
        self.assertEqual(normalize_extension(".HTM,L"), "html")

    def test_mimetype_extension(self):
        self.assertEqual(mime_ext(None), None)
        self.assertEqual(mime_ext(""), None)
        self.assertEqual(mime_ext("bla"), None)
        self.assertEqual(mime_ext("application/pdf"), "pdf")



from typing import Dict
from pantomime.types import DEFAULT, PLAIN, DIRECTORY
from pantomime.types import EXCEL, PDF, CSV, RTF, PSD, WORD
from pantomime.types import RAR, ZIP, GZIP, TIFF, JPEG

REPLACE: Dict[str, str] = {
    "x-unknown/unknown": DEFAULT,
    "unknown/unknown": DEFAULT,
    "x-unknown/octet-stream": DEFAULT,
    "application/x-unknown": DEFAULT,
    "file/unknown": DEFAULT,
    "content/unknown": DEFAULT,
    "application/x-unknown-application-octet-stream": DEFAULT,
    "x-unknown/stream": DEFAULT,
    "application/binary": DEFAULT,
    # Invalid
    "image/*": DEFAULT,
    "*/*": DEFAULT,
    # CSV
    "application/csv": CSV,
    # Plain
    "plain/text": PLAIN,
    "text/text": PLAIN,
    # Rich text
    "application/rtf": RTF,
    "file/rtf": RTF,
    "text/richtext": RTF,
    "text/enriched": RTF,
    # Word
    "application/x-msword": WORD,
    "application/msword": WORD,
    "application/vnd.ms-word.document.macroenabled.12": "application/vnd.ms-word.document.12",  # noqa
    # Excel normalisations:
    "application/x-excel": EXCEL,
    "application/x-msexcel": EXCEL,
    "application/excel": EXCEL,
    "application/x-ms-excel": EXCEL,
    "application/x-unknown-application-vnd.ms-excel": EXCEL,
    "application/vnd.ms-excel.12": EXCEL,
    # PDF
    "image/pdf": PDF,
    "application/x-pdf": PDF,
    "document/pdf": PDF,
    "x-application/apple-pdf": PDF,
    "file/pdf": PDF,
    "application/x-unknown-application-pdf": PDF,
    "text/pdf": PDF,
    "invalid/pdf": PDF,
    "application/vnd.pdf": PDF,
    "x-unknown/pdf": PDF,
    # RAR
    "application/x-rar": RAR,
    "application/x-rar-compressed": RAR,
    # Zip
    "application/x-zip": ZIP,
    "application/x-zip-compressed": ZIP,
    "appliation/zip": ZIP,
    # GZip
    "application/x-gzip": GZIP,
    "application/x-gzip-compressed": GZIP,
    "application/x-gunzip": GZIP,
    # JPEG
    "image/x-citrix-jpeg": JPEG,
    "application/jpeg": JPEG,
    "image/jpg": JPEG,
    "application/jpg": JPEG,
    # PSD
    "image/x-photoshop": PSD,
    "application/x-photoshop": PSD,
    "image/psd": PSD,
    # Tiff
    "image/x-tiff": TIFF,
    "application/tiff": TIFF,
    "application/x-tiff": TIFF,
    "text/x-vcard": "text/vcard",
    "text/directory": DIRECTORY,
    "application/x-msi": "application/vnd.ms-msi",
    "image/x.djvu": "image/vnd.djvu",
    "application/html": "text/html",
}



from setuptools import setup, find_packages

with open("README.md") as f:
    long_description = f.read()


setup(
    name="pantomime",
    version="0.6.0",
    description="MIME type normalisation and labels.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.6",
    ],
    keywords="mime mimetypes file types",
    author="Journalism Development Network, Inc.",
    author_email="data@occrp.org",
    url="http://github.com/alephdata/pantomime",
    license="MIT",
    packages=find_packages(exclude=["ez_setup", "examples", "test"]),
    namespace_packages=[],
    package_data={"pantomime": ["py.typed"]},
    include_package_data=True,
    zip_safe=False,
    install_requires=[
        "banal >= 0.3.5",
        "normality >= 0.5.6",
    ],
    tests_require=["nose"],
    entry_points={},
    extras_require={
        "dev": [
            "wheel>=0.29.0",
            "twine",
            "mypy",
            "black",
            "flake8>=2.6.0",
            "pytest",
            "pytest-cov",
            "banal",
            "coverage>=4.1",
        ]
    },
)



import unittest

from pantomime.filename import FileName


class FileNameTest(unittest.TestCase):
    def test_none_filename(self):
        fn = FileName(None)
        self.assertEqual(fn.file_name, None)
        self.assertEqual(fn.extension, None)
        self.assertFalse(fn.has_extension)
        self.assertEqual(fn.safe(), "data")

    def test_normal_filename(self):
        fn = FileName("testing .doc")
        self.assertEqual(fn.file_name, "testing .doc")
        self.assertEqual(fn.extension, "doc")
        self.assertTrue(fn.has_extension)
        self.assertEqual(fn.safe(), "testing.doc")
        self.assertEqual(fn.safe("xls"), "testing.xls")

    def test_no_ext_filename(self):
        fn = FileName("testing xxx")
        self.assertEqual(fn.extension, None)
        self.assertFalse(fn.has_extension)
        self.assertEqual(fn.safe(), "testing_xxx")
        self.assertEqual(fn.safe("doc"), "testing_xxx.doc")
